@@ -31,6 +31,7 @@ Element::Element()
 {
   this->dataPtr->copyChildren = false;
   this->dataPtr->referenceSDF = "";
+  this->dataPtr->explicitlySetInFile = true;
 }
 
 /////////////////////////////////////////////////
@@ -51,7 +52,7 @@ void Element::SetParent(const ElementPtr _parent)
 
   // If this element doesn't have a path, get it from the parent
   if (nullptr != _parent && (this->FilePath().empty() ||
-      this->FilePath() == "data-string"))
+      this->FilePath() == std::string(kSdfStringSource)))
   {
     this->SetFilePath(_parent->FilePath());
   }
@@ -93,6 +94,25 @@ void Element::SetCopyChildren(bool _value)
   this->dataPtr->copyChildren = _value;
 }
 
+/////////////////////////////////////////////////
+void Element::SetExplicitlySetInFile(const bool _value)
+{
+  this->dataPtr->explicitlySetInFile = _value;
+
+  ElementPtr_V::const_iterator eiter;
+  for (eiter = this->dataPtr->elements.begin();
+       eiter != this->dataPtr->elements.end(); ++eiter)
+  {
+    (*eiter)->SetExplicitlySetInFile(_value);
+  }
+}
+
+/////////////////////////////////////////////////
+bool Element::GetExplicitlySetInFile() const
+{
+  return this->dataPtr->explicitlySetInFile;
+}
+
 /////////////////////////////////////////////////
 bool Element::GetCopyChildren() const
 {
@@ -121,6 +141,19 @@ void Element::AddValue(const std::string &_type,
       _type, _defaultValue, _required, _description);
 }
 
+/////////////////////////////////////////////////
+void Element::AddValue(const std::string &_type,
+                       const std::string &_defaultValue,
+                       bool _required,
+                       const std::string &_minValue,
+                       const std::string &_maxValue,
+                       const std::string &_description)
+{
+  this->dataPtr->value =
+      std::make_shared<Param>(this->dataPtr->name, _type, _defaultValue,
+                              _required, _minValue, _maxValue, _description);
+}
+
 /////////////////////////////////////////////////
 ParamPtr Element::CreateParam(const std::string &_key,
                               const std::string &_type,
@@ -154,7 +187,10 @@ ElementPtr Element::Clone() const
   clone->dataPtr->includeFilename = this->dataPtr->includeFilename;
   clone->dataPtr->referenceSDF = this->dataPtr->referenceSDF;
   clone->dataPtr->path = this->dataPtr->path;
+  clone->dataPtr->lineNumber = this->dataPtr->lineNumber;
+  clone->dataPtr->xmlPath = this->dataPtr->xmlPath;
   clone->dataPtr->originalVersion = this->dataPtr->originalVersion;
+  clone->dataPtr->explicitlySetInFile = this->dataPtr->explicitlySetInFile;
 
   Param_V::const_iterator aiter;
   for (aiter = this->dataPtr->attributes.begin();
@@ -182,6 +218,11 @@ ElementPtr Element::Clone() const
     clone->dataPtr->value = this->dataPtr->value->Clone();
   }
 
+  if (this->dataPtr->includeElement)
+  {
+    clone->dataPtr->includeElement = this->dataPtr->includeElement->Clone();
+  }
+
   return clone;
 }
 
@@ -196,6 +237,9 @@ void Element::Copy(const ElementPtr _elem)
   this->dataPtr->referenceSDF = _elem->ReferenceSDF();
   this->dataPtr->originalVersion = _elem->OriginalVersion();
   this->dataPtr->path = _elem->FilePath();
+  this->dataPtr->lineNumber = _elem->LineNumber();
+  this->dataPtr->xmlPath = _elem->XmlPath();
+  this->dataPtr->explicitlySetInFile = _elem->GetExplicitlySetInFile();
 
   for (Param_V::iterator iter = _elem->dataPtr->attributes.begin();
        iter != _elem->dataPtr->attributes.end(); ++iter)
@@ -237,6 +281,18 @@ void Element::Copy(const ElementPtr _elem)
     elem->SetParent(shared_from_this());
     this->dataPtr->elements.push_back(elem);
   }
+
+  if (_elem->dataPtr->includeElement)
+  {
+    if (!this->dataPtr->includeElement)
+    {
+      this->dataPtr->includeElement = _elem->dataPtr->includeElement->Clone();
+    }
+    else
+    {
+      this->dataPtr->includeElement->Copy(_elem->dataPtr->includeElement);
+    }
+  }
 }
 
 /////////////////////////////////////////////////
@@ -251,6 +307,17 @@ void Element::PrintDescription(const std::string &_prefix) const
               << "'"
               << " default ='" << this->dataPtr->value->GetDefaultAsString()
               << "'";
+    auto minValue = this->dataPtr->value->GetMinValueAsString();
+    if (minValue.has_value())
+    {
+      std::cout << " min ='" << *minValue << "'";
+    }
+
+    auto maxValue = this->dataPtr->value->GetMaxValueAsString();
+    if (maxValue.has_value())
+    {
+      std::cout << " max ='" << *maxValue << "'";
+    }
   }
 
   std::cout << ">\n";
@@ -420,47 +487,59 @@ void Element::PrintDocLeftPane(std::string &_html, int _spacing,
 }
 
 void Element::PrintValuesImpl(const std::string &_prefix,
+                              bool _includeDefaultElements,
+                              bool _includeDefaultAttributes,
                               std::ostringstream &_out) const
 {
-  _out << _prefix << "<" << this->dataPtr->name;
+  if (this->GetExplicitlySetInFile() || _includeDefaultElements)
+  {
+    _out << _prefix << "<" << this->dataPtr->name;
 
-  Param_V::const_iterator aiter;
-  for (aiter = this->dataPtr->attributes.begin();
-       aiter != this->dataPtr->attributes.end(); ++aiter)
-  {
-    // Only print attribute values if they were set
-    // TODO(anyone): GetRequired is added here to support up-conversions where a
-    // new required attribute with a default value is added. We would have
-    // better separation of concerns if the conversion process set the required
-    // attributes with their default values.
-    if ((*aiter)->GetSet() || (*aiter)->GetRequired())
+    Param_V::const_iterator aiter;
+    for (aiter = this->dataPtr->attributes.begin();
+         aiter != this->dataPtr->attributes.end(); ++aiter)
     {
-      _out << " " << (*aiter)->GetKey() << "='"
-           << (*aiter)->GetAsString() << "'";
+      // Only print attribute values if they were set
+      // TODO(anyone): GetRequired is added here to support up-conversions where
+      // a new required attribute with a default value is added. We would have
+      // better separation of concerns if the conversion process set the
+      // required attributes with their default values.
+      if ((*aiter)->GetSet() || (*aiter)->GetRequired() ||
+          _includeDefaultAttributes)
+      {
+        _out << " " << (*aiter)->GetKey() << "='"
+             << (*aiter)->GetAsString() << "'";
+      }
     }
-  }
 
-  if (this->dataPtr->elements.size() > 0)
-  {
-    _out << ">\n";
-    ElementPtr_V::const_iterator eiter;
-    for (eiter = this->dataPtr->elements.begin();
-         eiter != this->dataPtr->elements.end(); ++eiter)
+    if (this->dataPtr->elements.size() > 0)
     {
-      (*eiter)->ToString(_prefix + "  ", _out);
-    }
-    _out << _prefix << "</" << this->dataPtr->name << ">\n";
-  }
-  else
-  {
-    if (this->dataPtr->value)
-    {
-      _out << ">" << this->dataPtr->value->GetAsString()
-           << "</" << this->dataPtr->name << ">\n";
+      _out << ">\n";
+      ElementPtr_V::const_iterator eiter;
+      for (eiter = this->dataPtr->elements.begin();
+           eiter != this->dataPtr->elements.end(); ++eiter)
+      {
+        if ((*eiter)->GetExplicitlySetInFile() || _includeDefaultElements)
+        {
+          (*eiter)->ToString(_prefix + "  ",
+                             _includeDefaultElements,
+                             _includeDefaultAttributes,
+                             _out);
+        }
+      }
+      _out << _prefix << "</" << this->dataPtr->name << ">\n";
     }
     else
     {
-      _out << "/>\n";
+      if (this->dataPtr->value)
+      {
+        _out << ">" << this->dataPtr->value->GetAsString()
+             << "</" << this->dataPtr->name << ">\n";
+      }
+      else
+      {
+        _out << "/>\n";
+      }
     }
   }
 }
@@ -469,7 +548,20 @@ void Element::PrintValuesImpl(const std::string &_prefix,
 void Element::PrintValues(std::string _prefix) const
 {
   std::ostringstream ss;
-  PrintValuesImpl(_prefix, ss);
+  PrintValuesImpl(_prefix, true, false, ss);
+  std::cout << ss.str();
+}
+
+/////////////////////////////////////////////////
+void Element::PrintValues(const std::string &_prefix,
+                          bool _includeDefaultElements,
+                          bool _includeDefaultAttributes) const
+{
+  std::ostringstream ss;
+  PrintValuesImpl(_prefix,
+                  _includeDefaultElements,
+                  _includeDefaultAttributes,
+                  ss);
   std::cout << ss.str();
 }
 
@@ -477,17 +569,35 @@ void Element::PrintValues(std::string _prefix) const
 std::string Element::ToString(const std::string &_prefix) const
 {
   std::ostringstream out;
-  this->ToString(_prefix, out);
+  this->ToString(_prefix, true, false, out);
+  return out.str();
+}
+
+/////////////////////////////////////////////////
+std::string Element::ToString(const std::string &_prefix,
+                              bool _includeDefaultElements,
+                              bool _includeDefaultAttributes) const
+{
+  std::ostringstream out;
+  this->ToString(_prefix,
+                 _includeDefaultElements,
+                 _includeDefaultAttributes,
+                 out);
   return out.str();
 }
 
 /////////////////////////////////////////////////
 void Element::ToString(const std::string &_prefix,
+                       bool _includeDefaultElements,
+                       bool _includeDefaultAttributes,
                        std::ostringstream &_out) const
 {
   if (this->dataPtr->includeFilename.empty())
   {
-    PrintValuesImpl(_prefix, _out);
+    PrintValuesImpl(_prefix,
+                    _includeDefaultElements,
+                    _includeDefaultAttributes,
+                    _out);
   }
   else
   {
@@ -515,6 +625,27 @@ bool Element::GetAttributeSet(const std::string &_key) const
   return result;
 }
 
+/////////////////////////////////////////////////
+void Element::RemoveAttribute(const std::string &_key)
+{
+  Param_V::const_iterator iter;
+  for (iter = this->dataPtr->attributes.begin();
+      iter != this->dataPtr->attributes.end(); ++iter)
+  {
+    if ((*iter)->GetKey() == _key)
+    {
+      this->dataPtr->attributes.erase(iter);
+      break;
+    }
+  }
+}
+
+/////////////////////////////////////////////////
+void Element::RemoveAllAttributes()
+{
+  this->dataPtr->attributes.clear();
+}
+
 /////////////////////////////////////////////////
 ParamPtr Element::GetAttribute(const std::string &_key) const
 {
@@ -741,6 +872,18 @@ ElementPtr Element::GetElement(const std::string &_name)
   return result;
 }
 
+/////////////////////////////////////////////////
+ElementPtr Element::FindElement(const std::string &_name)
+{
+  return this->GetElementImpl(_name);
+}
+
+/////////////////////////////////////////////////
+ElementConstPtr Element::FindElement(const std::string &_name) const
+{
+  return this->GetElementImpl(_name);
+}
+
 /////////////////////////////////////////////////
 void Element::InsertElement(ElementPtr _elem)
 {
@@ -805,6 +948,8 @@ void Element::Clear()
   this->ClearElements();
   this->dataPtr->originalVersion.clear();
   this->dataPtr->path.clear();
+  this->dataPtr->lineNumber = std::nullopt;
+  this->dataPtr->xmlPath.clear();
 }
 
 /////////////////////////////////////////////////
@@ -888,6 +1033,18 @@ std::string Element::GetInclude() const
   return this->dataPtr->includeFilename;
 }
 
+/////////////////////////////////////////////////
+void Element::SetIncludeElement(sdf::ElementPtr _includeElem)
+{
+  this->dataPtr->includeElement = _includeElem;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Element::GetIncludeElement() const
+{
+  return this->dataPtr->includeElement;
+}
+
 /////////////////////////////////////////////////
 void Element::SetFilePath(const std::string &_path)
 {
@@ -900,6 +1057,30 @@ const std::string &Element::FilePath() const
   return this->dataPtr->path;
 }
 
+/////////////////////////////////////////////////
+void Element::SetLineNumber(int _lineNumber)
+{
+  this->dataPtr->lineNumber = _lineNumber;
+}
+
+/////////////////////////////////////////////////
+std::optional<int> Element::LineNumber() const
+{
+  return this->dataPtr->lineNumber;
+}
+
+/////////////////////////////////////////////////
+void Element::SetXmlPath(const std::string &_path)
+{
+  this->dataPtr->xmlPath = _path;
+}
+
+/////////////////////////////////////////////////
+const std::string &Element::XmlPath() const
+{
+  return this->dataPtr->xmlPath;
+}
+
 /////////////////////////////////////////////////
 void Element::SetOriginalVersion(const std::string &_version)
 {
