@@ -16,12 +16,12 @@
 */
 
 #include <gtest/gtest.h>
-#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string>
 
-#include "sdf/Filesystem.hh"
+#include <ignition/utilities/ExtraTestMacros.hh>
+
 #include "sdf/parser.hh"
 #include "sdf/SDFImpl.hh"
 #include "sdf/sdf_config.h"
@@ -65,7 +65,7 @@ std::string custom_exec_str(std::string _cmd)
 }
 
 /////////////////////////////////////////////////
-TEST(check, SDF)
+TEST(check, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
 {
   std::string pathBase = PROJECT_SOURCE_PATH;
   pathBase += "/test/sdf";
@@ -99,7 +99,7 @@ TEST(check, SDF)
     // Check world_duplicate.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: World with name[default] already exists."),
+    EXPECT_NE(output.find("World with name[default] already exists."),
               std::string::npos) << output;
   }
 
@@ -111,17 +111,9 @@ TEST(check, SDF)
     // Check world_sibling_same_names.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: Non-unique names"), std::string::npos)
+    EXPECT_NE(output.find("Non-unique names"), std::string::npos)
       << output;
   }
-  // Check an SDF world file is allowed to have duplicate plugin names
-  {
-    std::string path = pathBase +"/world_duplicate_plugins.sdf";
-
-    std::string output =
-      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_EQ("Valid.\n", output) << output;
-  }
 
   // Check an SDF file with sibling elements of the same type (link)
   // that have duplicate names.
@@ -131,7 +123,7 @@ TEST(check, SDF)
     // Check model_duplicate_links.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: link with name[link] already exists."),
+    EXPECT_NE(output.find("link with name[link] already exists."),
               std::string::npos) << output;
   }
 
@@ -143,7 +135,7 @@ TEST(check, SDF)
     // Check model_duplicate_joints.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: joint with name[joint] already exists."),
+    EXPECT_NE(output.find("joint with name[joint] already exists."),
               std::string::npos) << output;
   }
 
@@ -155,19 +147,10 @@ TEST(check, SDF)
     // Check model_link_joint_same_name.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: Non-unique names"), std::string::npos)
+    EXPECT_NE(output.find("Non-unique names"), std::string::npos)
       << output;
   }
 
-  // Check an SDF model file is allowed to have duplicate plugin names
-  {
-    std::string path = pathBase +"/model_duplicate_plugins.sdf";
-
-    std::string output =
-      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_EQ("Valid.\n", output) << output;
-  }
-
   // Check an SDF file with sibling elements of the same type (collision)
   // that have duplicate names.
   {
@@ -176,7 +159,7 @@ TEST(check, SDF)
     // Check link_duplicate_sibling_collisions.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: collision with name[collision] "
+    EXPECT_NE(output.find("collision with name[collision] "
                           "already exists."),
               std::string::npos) << output;
   }
@@ -189,7 +172,7 @@ TEST(check, SDF)
     // Check link_duplicate_sibling_visuals.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: visual with name[visual] already exists."),
+    EXPECT_NE(output.find("visual with name[visual] already exists."),
               std::string::npos) << output;
   }
 
@@ -222,7 +205,7 @@ TEST(check, SDF)
     // Check joint_invalid_child.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: Child link with name[invalid] specified by "
+    EXPECT_NE(output.find("Child frame with name[invalid] specified by "
                           "joint with name[joint] not found in model with "
                           "name[joint_invalid_child]."),
               std::string::npos) << output;
@@ -235,12 +218,37 @@ TEST(check, SDF)
     // Check joint_invalid_parent.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: parent link with name[invalid] specified by "
+    EXPECT_NE(output.find("parent frame with name[invalid] specified by "
                           "joint with name[joint] not found in model with "
                           "name[joint_invalid_parent]."),
               std::string::npos) << output;
   }
 
+  // Check an SDF file with a joint that names itself as the child frame.
+  {
+    std::string path = pathBase +"/joint_invalid_self_child.sdf";
+
+    // Check joint_invalid_self_child.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(output.find("FrameAttachedToGraph cycle detected, already "
+                          "visited vertex [joint_invalid_self_child::self]."),
+              std::string::npos) << output;
+  }
+
+  // Check an SDF file with a joint that names itself as the parent frame.
+  {
+    std::string path = pathBase +"/joint_invalid_self_parent.sdf";
+
+    // Check joint_invalid_self_parent.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(output.find("joint with name[self] in model with "
+                          "name[joint_invalid_self_parent] must not specify "
+                          "its own name as the parent frame."),
+              std::string::npos) << output;
+  }
+
   // Check an SDF file with a joint with identical parent and child.
   {
     std::string path = pathBase +"/joint_invalid_parent_same_as_child.sdf";
@@ -248,12 +256,27 @@ TEST(check, SDF)
     // Check joint_invalid_parent_same_as_child.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: Joint with name[joint] must "
+    EXPECT_NE(output.find("Joint with name[joint] must "
                           "specify different link names for parent and child, "
                           "while [link] was specified for both."),
               std::string::npos) << output;
   }
 
+  // Check an SDF file with a joint with parent parent frame that resolves
+  // to the same value as the child.
+  {
+    std::string path =
+        pathBase + "/joint_invalid_resolved_parent_same_as_child.sdf";
+
+    // Check joint_invalid_resolved_parent_same_as_child.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(output.find("specified parent frame [J1] and child frame [L2] "
+                          "that both resolve to [L2], but they should resolve "
+                          "to different values."),
+              std::string::npos) << output;
+  }
+
   // Check an SDF file with the world specified as a child link.
   {
     std::string path = pathBase +"/joint_child_world.sdf";
@@ -261,7 +284,7 @@ TEST(check, SDF)
     // Check joint_child_world.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: Joint with name[joint] specified invalid "
+    EXPECT_NE(output.find("Joint with name[joint] specified invalid "
                           "child link [world]."),
               std::string::npos) << output;
   }
@@ -277,6 +300,28 @@ TEST(check, SDF)
     EXPECT_EQ("Valid.\n", output) << output;
   }
 
+  // Check an SDF file with a frame specified as the joint child.
+  // This is a valid file.
+  {
+    std::string path = pathBase +"/joint_child_frame.sdf";
+
+    // Check joint_child_frame.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
+
+  // Check an SDF file with a frame specified as the joint parent.
+  // This is a valid file.
+  {
+    std::string path = pathBase +"/joint_parent_frame.sdf";
+
+    // Check joint_parent_frame.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
+
   // Check an SDF file with the second link specified as the canonical link.
   // This is a valid file.
   {
@@ -295,7 +340,7 @@ TEST(check, SDF)
     // Check model_invalid_canonical_link.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: canonical_link with name[link3] not found in "
+    EXPECT_NE(output.find("canonical_link with name[link3] not found in "
                           "model with name[model_invalid_canonical_link]."),
               std::string::npos) << output;
   }
@@ -307,7 +352,7 @@ TEST(check, SDF)
     // Check model_without_links.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: A model must have at least one link."),
+    EXPECT_NE(output.find("A model must have at least one link."),
               std::string::npos) << output;
   }
 
@@ -335,15 +380,22 @@ TEST(check, SDF)
   // that is explicitly specified by //model/@canonical_link using ::
   // syntax.
   {
-    std::string path = pathBase +"/nested_invalid_explicit_canonical_link.sdf";
+    std::string path = pathBase +"/nested_explicit_canonical_link.sdf";
 
-    // Check nested_invalid_explicit_canonical_link.sdf
+    // Check nested_explicit_canonical_link.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: canonical_link with name[nested::link] not "
-                          "found in model with name[top]."),
-              std::string::npos) << output;
-    EXPECT_NE(output.find("Error: A model must have at least one link."),
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
+
+  // Check an SDF file with a model that a nested model without a link.
+  {
+    std::string path = pathBase +"/nested_without_links_invalid.sdf";
+
+    // Check nested_without_links_invalid.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(output.find("A model must have at least one link."),
               std::string::npos) << output;
   }
 
@@ -354,21 +406,21 @@ TEST(check, SDF)
     // Check model_invalid_reserved_names.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: The supplied link name [world] is reserved."),
+    EXPECT_NE(output.find("The supplied link name [world] is reserved."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: The supplied link name [__link__] "
+    EXPECT_NE(output.find("The supplied link name [__link__] "
                           "is reserved."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: The supplied visual name [__visual__] "
+    EXPECT_NE(output.find("The supplied visual name [__visual__] "
                           "is reserved."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: The supplied collision name [__collision__] "
+    EXPECT_NE(output.find("The supplied collision name [__collision__] "
                           "is reserved."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: The supplied joint name [__joint__] "
+    EXPECT_NE(output.find("The supplied joint name [__joint__] "
                           "is reserved."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: The supplied frame name [__frame__] "
+    EXPECT_NE(output.find("The supplied frame name [__frame__] "
                           "is reserved."),
               std::string::npos) << output;
   }
@@ -431,12 +483,12 @@ TEST(check, SDF)
     // Check model_frame_invalid_attached_to.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: attached_to name[A] specified by frame with "
+    EXPECT_NE(output.find("attached_to name[A] specified by frame with "
                           "name[F3] does not match a nested model, link, "
                           "joint, or frame name in model with "
                           "name[model_frame_invalid_attached_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: attached_to name[F4] is identical to frame "
+    EXPECT_NE(output.find("attached_to name[F4] is identical to frame "
                           "name[F4], causing a graph cycle in model with "
                           "name[model_frame_invalid_attached_to]."),
               std::string::npos) << output;
@@ -449,11 +501,13 @@ TEST(check, SDF)
     // Check model_frame_invalid_attached_to_cycle.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(std::string::npos, output.find(
-      "FrameAttachedToGraph cycle detected, already visited vertex [F1]."))
+    EXPECT_NE(std::string::npos,
+        output.find("FrameAttachedToGraph cycle detected, already visited "
+                    "vertex [model_frame_invalid_attached_to_cycle::F1]."))
         << output;
-    EXPECT_NE(std::string::npos, output.find(
-      "FrameAttachedToGraph cycle detected, already visited vertex [F2]."))
+    EXPECT_NE(std::string::npos,
+        output.find("FrameAttachedToGraph cycle detected, already visited "
+                    "vertex [model_frame_invalid_attached_to_cycle::F2]."))
         << output;
   }
 
@@ -475,12 +529,12 @@ TEST(check, SDF)
     // Check world_frame_invalid_attached_to.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: attached_to name[A] specified by frame with "
+    EXPECT_NE(output.find("attached_to name[A] specified by frame with "
                           "name[F] does not match a model or frame "
                           "name in world with "
                           "name[world_frame_invalid_attached_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: attached_to name[self_cycle] is identical "
+    EXPECT_NE(output.find("attached_to name[self_cycle] is identical "
                           "to frame name[self_cycle], causing a graph cycle "
                           "in world with "
                           "name[world_frame_invalid_attached_to]."),
@@ -505,12 +559,12 @@ TEST(check, SDF)
     // Check model_invalid_link_relative_to.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: relative_to name[A] specified by link with "
+    EXPECT_NE(output.find("relative_to name[A] specified by link with "
                           "name[L] does not match a nested model, link, "
                           "joint, or frame name in model with "
                           "name[model_invalid_link_relative_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: relative_to name[self_cycle] is identical to "
+    EXPECT_NE(output.find("relative_to name[self_cycle] is identical to "
                           "link name[self_cycle], causing a graph cycle in "
                           "model with name[model_invalid_link_relative_to]."),
               std::string::npos) << output;
@@ -527,18 +581,16 @@ TEST(check, SDF)
     EXPECT_EQ("Valid.\n", output) << output;
   }
 
-  // Check an invalid SDF file with a joint that specifies a child link
-  // within a sibling nested model using the unsupported :: syntax.
+  // Check an SDF file with nested_models using nested links/frames as joint
+  // parent or child frames.
+  // This is a valid file.
   {
-    std::string path = pathBase +"/model_invalid_nested_joint_child.sdf";
+    std::string path = pathBase +"/joint_nested_parent_child.sdf";
 
-    // Check model_invalid_nested_joint_child.sdf
+    // Check model_nested_model_relative_to.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: Child link with name[M::C] specified by "
-                          "joint with name[J] not found in model with "
-                          "name[model_invalid_nested_joint_child]."),
-              std::string::npos) << output;
+    EXPECT_EQ("Valid.\n", output) << output;
   }
 
   // Check an SDF file with joints using the relative_to attribute.
@@ -559,12 +611,12 @@ TEST(check, SDF)
     // Check model_invalid_joint_relative_to.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: relative_to name[A] specified by joint with "
+    EXPECT_NE(output.find("relative_to name[A] specified by joint with "
                           "name[J] does not match a nested model, link, "
                           "joint, or frame name in model with "
                           "name[model_invalid_joint_relative_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: relative_to name[Jcycle] is identical to "
+    EXPECT_NE(output.find("relative_to name[Jcycle] is identical to "
                           "joint name[Jcycle], causing a graph cycle in "
                           "model with name[model_invalid_joint_relative_to]."),
               std::string::npos) << output;
@@ -599,12 +651,12 @@ TEST(check, SDF)
     // Check model_invalid_frame_relative_to.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: relative_to name[A] specified by frame with "
+    EXPECT_NE(output.find("relative_to name[A] specified by frame with "
                           "name[F] does not match a nested model, link, "
                           "joint, or frame name in model with "
                           "name[model_invalid_frame_relative_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: relative_to name[cycle] is identical to "
+    EXPECT_NE(output.find("relative_to name[cycle] is identical to "
                           "frame name[cycle], causing a graph cycle in model "
                           "with name[model_invalid_frame_relative_to]."),
               std::string::npos) << output;
@@ -617,11 +669,13 @@ TEST(check, SDF)
     // Check model_invalid_frame_relative_to_cycle.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(std::string::npos, output.find(
-      "PoseRelativeToGraph cycle detected, already visited vertex [F1]."))
+    EXPECT_NE(std::string::npos,
+        output.find("PoseRelativeToGraph cycle detected, already visited "
+                    "vertex [model_invalid_frame_relative_to_cycle::F1]."))
         << output;
-    EXPECT_NE(std::string::npos, output.find(
-      "PoseRelativeToGraph cycle detected, already visited vertex [F2]."))
+    EXPECT_NE(std::string::npos,
+        output.find("PoseRelativeToGraph cycle detected, already visited "
+                    "vertex [model_invalid_frame_relative_to_cycle::F2]."))
         << output;
   }
 
@@ -643,31 +697,163 @@ TEST(check, SDF)
     // Check world_frame_invalid_relative_to.sdf
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
-    EXPECT_NE(output.find("Error: relative_to name[A] specified by model with "
+    EXPECT_NE(output.find("relative_to name[A] specified by model with "
                           "name[M] does not match a model or frame "
                           "name in world with "
                           "name[world_frame_invalid_relative_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: relative_to name[cycle] is identical "
+    EXPECT_NE(output.find("relative_to name[cycle] is identical "
                           "to model name[cycle], causing a graph cycle "
                           "in world with "
                           "name[world_frame_invalid_relative_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: relative_to name[A] specified by frame with "
+    EXPECT_NE(output.find("relative_to name[A] specified by frame with "
                           "name[F] does not match a model or frame "
                           "name in world with "
                           "name[world_frame_invalid_relative_to]."),
               std::string::npos) << output;
-    EXPECT_NE(output.find("Error: relative_to name[self_cycle] is identical "
+    EXPECT_NE(output.find("relative_to name[self_cycle] is identical "
                           "to frame name[self_cycle], causing a graph cycle "
                           "in world with "
                           "name[world_frame_invalid_relative_to]."),
               std::string::npos) << output;
   }
+  // Check an SDF file with an invalid frame specified as the placement_frame
+  // attribute.
+  {
+    std::string path = pathBase + "/model_invalid_placement_frame.sdf";
+
+    // Check model_invalid_placement_frame.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(
+        output.find("unable to find unique frame with name [link3] in graph"),
+        std::string::npos)
+        << output;
+  }
+
+  // Check an SDF file with an valid nested model cross references
+  {
+    std::string path = pathBase + "/nested_model_cross_references.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
+
+  // Check an SDF model file with an invalid usage of __root__
+  {
+    std::string path = pathBase + "/model_invalid_root_reference.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a "
+                    "value of attribute [relative_to]"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a"
+                    " value of attribute [attached_to]"),
+        std::string::npos)
+        << output;
+  }
+  // Check an SDF world file with an invalid usage of __root__
+  {
+    // Set SDF_PATH so that included models can be found
+    sdf::testing::setenv(
+      "SDF_PATH", sdf::testing::SourceFile("test", "integration", "model"));
+    std::string path = pathBase + "/world_invalid_root_reference.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a "
+                    "value of attribute [relative_to]"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a "
+                    "value of attribute [attached_to]"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a "
+                    "value of attribute [placement_frame]"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a "
+                    "value of attribute [canonical_link]"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a "
+                    "value of attribute [parent_frame]"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find("'__root__' is reserved; it cannot be used as a "
+                    "value of element [placement_frame]"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find(
+            "The supplied joint child name [__root__] is not valid"),
+        std::string::npos)
+        << output;
+    EXPECT_NE(
+        output.find(
+            "The supplied joint parent name [__root__] is not valid"),
+        std::string::npos)
+        << output;
+  }
+  // Check an SDF world file with an valid usage of __root__
+  {
+    std::string path = pathBase + "/world_valid_root_reference.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
+  // Check an SDF with an invalid relative frame at the top level model
+  {
+    std::string path = pathBase + "/model_invalid_top_level_frame.sdf";
+
+    std::string output =
+        custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_NE(
+        output.find("Attribute //pose[@relative_to] of top level model must be "
+                    "left empty, found //pose[@relative_to='some_frame']."),
+        std::string::npos) << output;
+  }
 }
 
 /////////////////////////////////////////////////
-TEST(check_model_sdf, SDF)
+TEST(check_shapes_sdf, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  std::string pathBase = PROJECT_SOURCE_PATH;
+  pathBase += "/test/sdf";
+
+  {
+    std::string path = pathBase +"/shapes.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output);
+  }
+
+  {
+    std::string path = pathBase +"/shapes_world.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output);
+  }
+}
+
+/////////////////////////////////////////////////
+TEST(check_model_sdf, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
 {
   std::string pathBase = PROJECT_SOURCE_PATH;
   pathBase += "/test/integration/model/box";
@@ -693,7 +879,7 @@ TEST(check_model_sdf, SDF)
 }
 
 /////////////////////////////////////////////////
-TEST(describe, SDF)
+TEST(describe, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
 {
   // Get the description
   std::string output =
@@ -705,7 +891,7 @@ TEST(describe, SDF)
 }
 
 /////////////////////////////////////////////////
-TEST(print, SDF)
+TEST(print, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
 {
   std::string pathBase = PROJECT_SOURCE_PATH;
   pathBase += "/test/sdf";
@@ -735,118 +921,225 @@ TEST(print, SDF)
 }
 
 /////////////////////////////////////////////////
-TEST(inertial_stats, SDF)
+TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(WorldPoseRelativeTo))
 {
-  std::string pathBase = PROJECT_SOURCE_PATH;
-  pathBase += "/test/sdf";
+  const std::string pathBase = std::string(PROJECT_SOURCE_PATH) + "/test/sdf";
 
-  auto expectedOutput =
-    "Inertial statistics for model: test_model\n"
-    "---\n"
-    "Total mass of the model: 24\n"
-    "---\n"
-    "Centre of mass in model frame: \n"
-    "X: 0\n"
-    "Y: 0\n"
-    "Z: 0\n"
-    "---\n"
-    "Moment of inertia matrix: \n"
-    "304  0    0    \n"
-    "0    304  0    \n"
-    "0    0    604  \n"
-    "---\n";
+  // world pose relative_to graph
+  const std::string path =
+    pathBase + "/world_relative_to_nested_reference.sdf";
 
-  // Check a good SDF file by passing the absolute path
-  {
-    std::string path = pathBase +"/inertial_stats.sdf";
+  const std::string output =
+    custom_exec_str(IgnCommand() + " sdf -g pose " + path + SdfVersion());
 
-    std::string output =
-      custom_exec_str(IgnCommand() + " sdf --inertial-stats " +
-                      path + SdfVersion());
-    EXPECT_EQ(expectedOutput, output);
-  }
-
-  // Check a good SDF file from the same folder by passing a relative path
-  {
-    std::string path = "inertial_stats.sdf";
-
-    std::string output =
-      custom_exec_str("cd " + pathBase + " && " +
-                      IgnCommand() + " sdf --inertial-stats " +
-                      path + SdfVersion());
-    EXPECT_EQ(expectedOutput, output);
-  }
-
-  expectedOutput =
-          "Error: A link named link has invalid inertia.\n"
-          "Inertial statistics for model: model\n"
-          "---\n"
-          "Total mass of the model: 0\n"
-          "---\n"
-          "Centre of mass in model frame: \n"
-          "X: 0\n"
-          "Y: 0\n"
-          "Z: 0\n"
-          "---\n"
-          "Moment of inertia matrix: \n"
-          "0  0  0  \n"
-          "0  0  0  \n"
-          "0  0  0  \n"
-          "---\n";
-
-  // Check an invalid SDF file by passing the absolute path
-  {
-    std::string path = pathBase +"/inertial_invalid.sdf";
-
-    std::string output =
-      custom_exec_str(IgnCommand() + " sdf --inertial-stats " +
-                      path + SdfVersion());
-    EXPECT_EQ(expectedOutput, output);
-  }
-
-  expectedOutput =
-          "Error: Expected a model file but received a world file.\n";
-  // Check a valid world file.
-  {
-    std::string path = pathBase +"/box_plane_low_friction_test.world";
-
-    std::string output =
-      custom_exec_str(IgnCommand() + " sdf --inertial-stats " +
-                      path + SdfVersion());
-    EXPECT_EQ(expectedOutput, output);
-  }
+  std::stringstream expected;
+  expected << "digraph {\n"
+    << "  0 [label=\"__root__ (0)\"];\n"
+    << "  1 [label=\"world (1)\"];\n"
+    << "  2 [label=\"M1 (2)\"];\n"
+    << "  3 [label=\"M1::__model__ (3)\"];\n"
+    << "  4 [label=\"M1::L1 (4)\"];\n"
+    << "  5 [label=\"M1::L2 (5)\"];\n"
+    << "  6 [label=\"M1::J1 (6)\"];\n"
+    << "  7 [label=\"M1::F1 (7)\"];\n"
+    << "  8 [label=\"M1::CM1 (8)\"];\n"
+    << "  9 [label=\"M1::CM1::__model__ (9)\"];\n"
+    << "  10 [label=\"M1::CM1::L (10)\"];\n"
+    << "  11 [label=\"F2 (11)\"];\n"
+    << "  12 [label=\"F3 (12)\"];\n"
+    << "  13 [label=\"F4 (13)\"];\n"
+    << "  14 [label=\"F5 (14)\"];\n"
+    << "  15 [label=\"F6 (15)\"];\n"
+    << "  16 [label=\"F7 (16)\"];\n"
+    << "  0 -> 1 [label=1];\n"
+    << "  2 -> 3 [label=0];\n"
+    << "  3 -> 4 [label=1];\n"
+    << "  3 -> 5 [label=1];\n"
+    << "  8 -> 9 [label=0];\n"
+    << "  9 -> 10 [label=1];\n"
+    << "  5 -> 6 [label=1];\n"
+    << "  5 -> 7 [label=1];\n"
+    << "  3 -> 8 [label=1];\n"
+    << "  1 -> 2 [label=1];\n"
+    << "  2 -> 11 [label=1];\n"
+    << "  4 -> 12 [label=1];\n"
+    << "  6 -> 13 [label=1];\n"
+    << "  7 -> 14 [label=1];\n"
+    << "  8 -> 15 [label=1];\n"
+    << "  10 -> 16 [label=1];\n"
+    << "}";
+  EXPECT_EQ(sdf::trim(expected.str()), sdf::trim(output));
 }
 
-//////////////////////////////////////////////////
-/// \brief Check help message and bash completion script for consistent flags
-TEST(HelpVsCompletionFlags, SDF)
+/////////////////////////////////////////////////
+TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelPoseRelativeTo))
 {
-  // Flags in help message
-  std::string helpOutput = custom_exec_str(IgnCommand() + " sdf --help");
+  const std::string pathBase = std::string(PROJECT_SOURCE_PATH) + "/test/sdf";
+  const std::string path = pathBase + "/model_relative_to_nested_reference.sdf";
+  const std::string output =
+    custom_exec_str(IgnCommand() + " sdf -g pose " + path + SdfVersion());
 
-  // Call the output function in the bash completion script
-  std::string scriptPath = PROJECT_SOURCE_PATH;
-  scriptPath = sdf::filesystem::append(scriptPath, "src", "cmd",
-      "sdf.bash_completion.sh");
+  std::stringstream expected;
+  expected << "digraph {\n"
+    << "  0 [label=\"__root__ (0)\"];\n"
+    << "  1 [label=\"parent_model (1)\"];\n"
+    << "  2 [label=\"parent_model::__model__ (2)\"];\n"
+    << "  3 [label=\"parent_model::L (3)\"];\n"
+    << "  4 [label=\"parent_model::M1 (4)\"];\n"
+    << "  5 [label=\"parent_model::M1::__model__ (5)\"];\n"
+    << "  6 [label=\"parent_model::M1::L1 (6)\"];\n"
+    << "  7 [label=\"parent_model::M1::L2 (7)\"];\n"
+    << "  8 [label=\"parent_model::M1::J1 (8)\"];\n"
+    << "  9 [label=\"parent_model::M1::F1 (9)\"];\n"
+    << "  10 [label=\"parent_model::M1::CM1 (10)\"];\n"
+    << "  11 [label=\"parent_model::M1::CM1::__model__ (11)\"];\n"
+    << "  12 [label=\"parent_model::M1::CM1::L (12)\"];\n"
+    << "  13 [label=\"parent_model::M2 (13)\"];\n"
+    << "  14 [label=\"parent_model::M2::__model__ (14)\"];\n"
+    << "  15 [label=\"parent_model::M2::L (15)\"];\n"
+    << "  16 [label=\"parent_model::M3 (16)\"];\n"
+    << "  17 [label=\"parent_model::M3::__model__ (17)\"];\n"
+    << "  18 [label=\"parent_model::M3::L (18)\"];\n"
+    << "  19 [label=\"parent_model::M4 (19)\"];\n"
+    << "  20 [label=\"parent_model::M4::__model__ (20)\"];\n"
+    << "  21 [label=\"parent_model::M4::L (21)\"];\n"
+    << "  22 [label=\"parent_model::M5 (22)\"];\n"
+    << "  23 [label=\"parent_model::M5::__model__ (23)\"];\n"
+    << "  24 [label=\"parent_model::M5::L (24)\"];\n"
+    << "  25 [label=\"parent_model::M6 (25)\"];\n"
+    << "  26 [label=\"parent_model::M6::__model__ (26)\"];\n"
+    << "  27 [label=\"parent_model::M6::L (27)\"];\n"
+    << "  28 [label=\"parent_model::M7 (28)\"];\n"
+    << "  29 [label=\"parent_model::M7::__model__ (29)\"];\n"
+    << "  30 [label=\"parent_model::M7::L (30)\"];\n"
+    << "  1 -> 2 [label=0];\n"
+    << "  2 -> 3 [label=1];\n"
+    << "  4 -> 5 [label=0];\n"
+    << "  5 -> 6 [label=1];\n"
+    << "  5 -> 7 [label=1];\n"
+    << "  10 -> 11 [label=0];\n"
+    << "  11 -> 12 [label=1];\n"
+    << "  7 -> 8 [label=1];\n"
+    << "  7 -> 9 [label=1];\n"
+    << "  5 -> 10 [label=1];\n"
+    << "  13 -> 14 [label=0];\n"
+    << "  14 -> 15 [label=1];\n"
+    << "  16 -> 17 [label=0];\n"
+    << "  17 -> 18 [label=1];\n"
+    << "  19 -> 20 [label=0];\n"
+    << "  20 -> 21 [label=1];\n"
+    << "  22 -> 23 [label=0];\n"
+    << "  23 -> 24 [label=1];\n"
+    << "  25 -> 26 [label=0];\n"
+    << "  26 -> 27 [label=1];\n"
+    << "  28 -> 29 [label=0];\n"
+    << "  29 -> 30 [label=1];\n"
+    << "  2 -> 4 [label=1];\n"
+    << "  4 -> 13 [label=1];\n"
+    << "  6 -> 16 [label=1];\n"
+    << "  8 -> 19 [label=1];\n"
+    << "  9 -> 22 [label=1];\n"
+    << "  10 -> 25 [label=1];\n"
+    << "  12 -> 28 [label=1];\n"
+    << "  0 -> 1 [label=1];\n"
+    << "}";
 
-  // Equivalent to:
-  // sh -c "bash -c \". /path/to/sdf.bash_completion.sh; _gz_sdf_flags\""
-  std::string cmd = "bash -c \". " + scriptPath +
-    "; _gz_sdf_flags\"";
-  std::string scriptOutput = custom_exec_str(cmd);
+  EXPECT_EQ(sdf::trim(expected.str()), sdf::trim(output));
+}
 
-  // Tokenize script output
-  std::istringstream iss(scriptOutput);
-  std::vector<std::string> flags((std::istream_iterator<std::string>(iss)),
-    std::istream_iterator<std::string>());
+/////////////////////////////////////////////////
+TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(WorldFrameAttachedTo))
+{
+  const std::string pathBase = std::string(PROJECT_SOURCE_PATH) + "/test/sdf";
+  const std::string path = pathBase + "/world_nested_frame_attached_to.sdf";
+  const std::string output =
+      custom_exec_str(IgnCommand() + " sdf -g frame " + path + SdfVersion());
 
-  EXPECT_GT(flags.size(), 0u);
+  std::stringstream expected;
 
-  // Match each flag in script output with help message
-  for (const auto &flag : flags)
-  {
-    EXPECT_NE(std::string::npos, helpOutput.find(flag)) << helpOutput;
-  }
+  expected << "digraph {\n"
+           << "  0 [label=\"world (0)\"];\n"
+           << "  1 [label=\"M1 (1)\"];\n"
+           << "  2 [label=\"M1::__model__ (2)\"];\n"
+           << "  3 [label=\"M1::L (3)\"];\n"
+           << "  4 [label=\"M1::J1 (4)\"];\n"
+           << "  5 [label=\"M1::F0 (5)\"];\n"
+           << "  6 [label=\"M1::M2 (6)\"];\n"
+           << "  7 [label=\"M1::M2::__model__ (7)\"];\n"
+           << "  8 [label=\"M1::M2::L (8)\"];\n"
+           << "  9 [label=\"world_frame (9)\"];\n"
+           << "  10 [label=\"F0 (10)\"];\n"
+           << "  11 [label=\"F1 (11)\"];\n"
+           << "  12 [label=\"F2 (12)\"];\n"
+           << "  13 [label=\"F3 (13)\"];\n"
+           << "  14 [label=\"F4 (14)\"];\n"
+           << "  15 [label=\"F5 (15)\"];\n"
+           << "  16 [label=\"F6 (16)\"];\n"
+           << "  1 -> 2 [label=0];\n"
+           << "  6 -> 7 [label=0];\n"
+           << "  7 -> 8 [label=1];\n"
+           << "  4 -> 5 [label=1];\n"
+           << "  5 -> 8 [label=1];\n"
+           << "  2 -> 3 [label=1];\n"
+           << "  9 -> 0 [label=1];\n"
+           << "  10 -> 0 [label=1];\n"
+           << "  11 -> 1 [label=1];\n"
+           << "  12 -> 3 [label=1];\n"
+           << "  13 -> 6 [label=1];\n"
+           << "  14 -> 8 [label=1];\n"
+           << "  15 -> 5 [label=1];\n"
+           << "  16 -> 4 [label=1];\n"
+           << "}";
+  EXPECT_EQ(sdf::trim(expected.str()), sdf::trim(output));
+}
+
+/////////////////////////////////////////////////
+TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelFrameAttachedTo))
+{
+  const std::string pathBase = std::string(PROJECT_SOURCE_PATH) + "/test/sdf";
+  const std::string path = pathBase + "/model_nested_frame_attached_to.sdf";
+  const std::string output =
+      custom_exec_str(IgnCommand() + " sdf -g frame " + path + SdfVersion());
+
+  std::stringstream expected;
+
+  expected
+      << "digraph {\n"
+      << "  0 [label=\"__root__ (0)\"];\n"
+      << "  1 [label=\"nested_model_frame_attached_to (1)\"];\n"
+      << "  2 [label=\"nested_model_frame_attached_to::__model__ (2)\"];\n"
+      << "  3 [label=\"nested_model_frame_attached_to::L (3)\"];\n"
+      << "  4 [label=\"nested_model_frame_attached_to::F0 (4)\"];\n"
+      << "  5 [label=\"nested_model_frame_attached_to::F1 (5)\"];\n"
+      << "  6 [label=\"nested_model_frame_attached_to::F2 (6)\"];\n"
+      << "  7 [label=\"nested_model_frame_attached_to::F3 (7)\"];\n"
+      << "  8 [label=\"nested_model_frame_attached_to::F4 (8)\"];\n"
+      << "  9 [label=\"nested_model_frame_attached_to::M1 (9)\"];\n"
+      << "  10 [label=\"nested_model_frame_attached_to::M1::__model__ "
+         "(10)\"];\n"
+      << "  11 [label=\"nested_model_frame_attached_to::M1::L (11)\"];\n"
+      << "  12 [label=\"nested_model_frame_attached_to::M1::J1 (12)\"];\n"
+      << "  13 [label=\"nested_model_frame_attached_to::M1::F (13)\"];\n"
+      << "  14 [label=\"nested_model_frame_attached_to::M1::M2 (14)\"];\n"
+      << "  15 [label=\"nested_model_frame_attached_to::M1::M2::__model__ "
+         "(15)\"];\n"
+      << "  16 [label=\"nested_model_frame_attached_to::M1::M2::L (16)\"];\n"
+      << "  1 -> 2 [label=0];\n"
+      << "  9 -> 10 [label=0];\n"
+      << "  14 -> 15 [label=0];\n"
+      << "  15 -> 16 [label=1];\n"
+      << "  12 -> 13 [label=1];\n"
+      << "  13 -> 14 [label=1];\n"
+      << "  10 -> 11 [label=1];\n"
+      << "  4 -> 9 [label=1];\n"
+      << "  5 -> 11 [label=1];\n"
+      << "  6 -> 5 [label=1];\n"
+      << "  7 -> 13 [label=1];\n"
+      << "  8 -> 12 [label=1];\n"
+      << "  2 -> 3 [label=1];\n"
+      << "}";
+
+  EXPECT_EQ(sdf::trim(expected.str()), sdf::trim(output));
 }
 
 /////////////////////////////////////////////////
