@@ -42,13 +42,15 @@ TEST(Frame, ModelFrame)
   std::string version = SDF_VERSION;
   stream
     << "<sdf version='" << version << "'>"
-    << "<model name='my_model'>"
-    << "  <frame name='mframe'>"
-    << "    <pose relative_to='/world'>1 1 0 0 0 0</pose>"
-    << "  </frame>"
-    << "  <pose relative_to='mframe'>1 0 0 0 0 0</pose>"
-    << "  <link name='link'/>"
-    << "</model>"
+    << "<world name='default'>"
+    << "  <model name='my_model'>"
+    << "    <frame name='mframe'>"
+    << "      <pose relative_to='/world'>1 1 0 0 0 0</pose>"
+    << "    </frame>"
+    << "    <pose relative_to='mframe'>1 0 0 0 0 0</pose>"
+    << "    <link name='link'/>"
+    << "  </model>"
+    << "</world>"
     << "</sdf>";
 
   sdf::SDFPtr sdfParsed(new sdf::SDF());
@@ -57,9 +59,15 @@ TEST(Frame, ModelFrame)
 
   // Verify correct parsing
 
+  // world
+  ASSERT_TRUE(sdfParsed->Root()->HasElement("world"));
+  sdf::ElementPtr worldElem = sdfParsed->Root()->GetElement("world");
+  ASSERT_NE(worldElem, nullptr);
+
   // model
-  EXPECT_TRUE(sdfParsed->Root()->HasElement("model"));
-  sdf::ElementPtr modelElem = sdfParsed->Root()->GetElement("model");
+  ASSERT_TRUE(worldElem->HasElement("model"));
+  sdf::ElementPtr modelElem = worldElem->GetElement("model");
+  ASSERT_TRUE(modelElem);
   EXPECT_TRUE(modelElem->HasAttribute("name"));
   EXPECT_EQ(modelElem->Get<std::string>("name"), "my_model");
 
@@ -74,16 +82,16 @@ TEST(Frame, ModelFrame)
   sdf::ElementPtr poseElem = frameElem->GetElement("pose");
   EXPECT_TRUE(poseElem->HasAttribute("relative_to"));
   EXPECT_EQ(poseElem->Get<std::string>("relative_to"), "/world");
-  EXPECT_EQ(poseElem->Get<gz::math::Pose3d>(),
-            gz::math::Pose3d(1, 1, 0, 0, 0, 0));
+  EXPECT_EQ(poseElem->Get<ignition::math::Pose3d>(),
+            ignition::math::Pose3d(1, 1, 0, 0, 0, 0));
 
   // model pose
   EXPECT_TRUE(modelElem->HasElement("pose"));
   sdf::ElementPtr modelPoseElem = modelElem->GetElement("pose");
   EXPECT_TRUE(modelPoseElem->HasAttribute("relative_to"));
   EXPECT_EQ(modelPoseElem->Get<std::string>("relative_to"), "mframe");
-  EXPECT_EQ(modelPoseElem->Get<gz::math::Pose3d>(),
-            gz::math::Pose3d(1, 0, 0, 0, 0, 0));
+  EXPECT_EQ(modelPoseElem->Get<ignition::math::Pose3d>(),
+            ignition::math::Pose3d(1, 0, 0, 0, 0, 0));
 
   // link
   EXPECT_TRUE(modelElem->HasElement("link"));
@@ -129,8 +137,8 @@ TEST(Frame, FrameDefaultPose)
   sdf::ElementPtr poseElem = frameElem->GetElement("pose");
   EXPECT_TRUE(poseElem->HasAttribute("relative_to"));
   EXPECT_EQ(poseElem->Get<std::string>("relative_to"), "");
-  EXPECT_EQ(poseElem->Get<gz::math::Pose3d>(),
-            gz::math::Pose3d(0, 0, 0, 0, 0, 0));
+  EXPECT_EQ(poseElem->Get<ignition::math::Pose3d>(),
+            ignition::math::Pose3d(0, 0, 0, 0, 0, 0));
 
   // link
   EXPECT_TRUE(modelElem->HasElement("link"));
@@ -176,8 +184,8 @@ TEST(Frame, NoFrame)
     sdf::ElementPtr poseElem = frameElem->GetElement("pose");
     EXPECT_TRUE(poseElem->HasAttribute("relative_to"));
     EXPECT_EQ(poseElem->Get<std::string>("relative_to"), "");
-    EXPECT_EQ(poseElem->Get<gz::math::Pose3d>(),
-              gz::math::Pose3d(0, 0, 0, 0, 0, 0));
+    EXPECT_EQ(poseElem->Get<ignition::math::Pose3d>(),
+              ignition::math::Pose3d(0, 0, 0, 0, 0, 0));
   }
 
   // link
@@ -192,8 +200,8 @@ TEST(Frame, NoFrame)
     sdf::ElementPtr poseElem = linkElem->GetElement("pose");
     EXPECT_TRUE(poseElem->HasAttribute("relative_to"));
     EXPECT_EQ(poseElem->Get<std::string>("relative_to"), "");
-    EXPECT_EQ(poseElem->Get<gz::math::Pose3d>(),
-              gz::math::Pose3d(0, 0, 0, 0, 0, 0));
+    EXPECT_EQ(poseElem->Get<ignition::math::Pose3d>(),
+              ignition::math::Pose3d(0, 0, 0, 0, 0, 0));
   }
 }
 
@@ -250,16 +258,16 @@ TEST(Frame, StateFrame)
     sdf::ElementPtr poseElem = frameElem->GetElement("pose");
     EXPECT_TRUE(poseElem->HasAttribute("relative_to"));
     EXPECT_EQ(poseElem->Get<std::string>("relative_to"), "/world");
-    EXPECT_EQ(poseElem->Get<gz::math::Pose3d>(),
-              gz::math::Pose3d(1, 0, 2, 0, 0, 0));
+    EXPECT_EQ(poseElem->Get<ignition::math::Pose3d>(),
+              ignition::math::Pose3d(1, 0, 2, 0, 0, 0));
 
     // model pose
     EXPECT_TRUE(modelStateElem->HasElement("pose"));
     sdf::ElementPtr modelPoseElem = modelStateElem->GetElement("pose");
     EXPECT_TRUE(modelPoseElem->HasAttribute("relative_to"));
     EXPECT_EQ(modelPoseElem->Get<std::string>("relative_to"), "mframe");
-    EXPECT_EQ(modelPoseElem->Get<gz::math::Pose3d>(),
-              gz::math::Pose3d(3, 3, 9, 0, 0, 0));
+    EXPECT_EQ(modelPoseElem->Get<ignition::math::Pose3d>(),
+              ignition::math::Pose3d(3, 3, 9, 0, 0, 0));
   }
 
   // link
@@ -274,8 +282,8 @@ TEST(Frame, StateFrame)
     sdf::ElementPtr linkPoseElem = linkStateElem->GetElement("pose");
     EXPECT_TRUE(linkPoseElem->HasAttribute("relative_to"));
     EXPECT_EQ(linkPoseElem->Get<std::string>("relative_to"), "lframe");
-    EXPECT_EQ(linkPoseElem->Get<gz::math::Pose3d>(),
-              gz::math::Pose3d(111, 3, 0, 0, 0, 0));
+    EXPECT_EQ(linkPoseElem->Get<ignition::math::Pose3d>(),
+              ignition::math::Pose3d(111, 3, 0, 0, 0, 0));
   }
 
   EXPECT_TRUE(stateElem->HasElement("light"));
@@ -291,8 +299,8 @@ TEST(Frame, StateFrame)
     sdf::ElementPtr lightPoseElem = lightStateElem->GetElement("pose");
     EXPECT_TRUE(lightPoseElem->HasAttribute("relative_to"));
     EXPECT_EQ(lightPoseElem->Get<std::string>("relative_to"), "lframe");
-    EXPECT_EQ(lightPoseElem->Get<gz::math::Pose3d>(),
-              gz::math::Pose3d(99, 0, 22, 0, 0, 0));
+    EXPECT_EQ(lightPoseElem->Get<ignition::math::Pose3d>(),
+              ignition::math::Pose3d(99, 0, 22, 0, 0, 0));
   }
 }
 
@@ -337,8 +345,8 @@ TEST(Frame, IncludeRelativeTo)
   sdf::ElementPtr modelPoseElem = modelElem->GetElement("pose");
   EXPECT_TRUE(modelPoseElem->HasAttribute("relative_to"));
   EXPECT_EQ(modelPoseElem->Get<std::string>("relative_to"), "/world");
-  EXPECT_EQ(modelPoseElem->Get<gz::math::Pose3d>(),
-            gz::math::Pose3d(5, -2, 1, 0, 0, 0));
+  EXPECT_EQ(modelPoseElem->Get<ignition::math::Pose3d>(),
+            ignition::math::Pose3d(5, -2, 1, 0, 0, 0));
 }
 
 ////////////////////////////////////////
@@ -389,8 +397,8 @@ TEST(Frame, IncludeRelativeToEmptyPose)
     sdf::ElementPtr modelPoseElem = modelElem->GetElement("pose");
     EXPECT_TRUE(modelPoseElem->HasAttribute("relative_to"));
     EXPECT_EQ(modelPoseElem->Get<std::string>("relative_to"), "/world");
-    EXPECT_EQ(modelPoseElem->Get<gz::math::Pose3d>(),
-        gz::math::Pose3d::Zero);
+    EXPECT_EQ(modelPoseElem->Get<ignition::math::Pose3d>(),
+        ignition::math::Pose3d::Zero);
   }
   // Check next model
   modelElem = modelElem->GetNextElement("model");
@@ -404,8 +412,8 @@ TEST(Frame, IncludeRelativeToEmptyPose)
     sdf::ElementPtr modelPoseElem = modelElem->GetElement("pose");
     EXPECT_TRUE(modelPoseElem->HasAttribute("relative_to"));
     EXPECT_FALSE(modelPoseElem->GetAttribute("relative_to")->GetSet());
-    EXPECT_EQ(modelPoseElem->Get<gz::math::Pose3d>(),
-        gz::math::Pose3d::Zero);
+    EXPECT_EQ(modelPoseElem->Get<ignition::math::Pose3d>(),
+        ignition::math::Pose3d::Zero);
   }
 }
 
@@ -420,13 +428,13 @@ TEST(DOMFrame, LoadModelFramesAttachedTo)
   EXPECT_TRUE(root.Load(testFile).empty());
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_attached_to", model->Name());
   EXPECT_EQ(1u, model->LinkCount());
   EXPECT_NE(nullptr, model->LinkByIndex(0));
   EXPECT_EQ(nullptr, model->LinkByIndex(1));
-  EXPECT_EQ(gz::math::Pose3d(0, 0, 0, 0, 0, 0), model->RawPose());
+  EXPECT_EQ(ignition::math::Pose3d(0, 0, 0, 0, 0, 0), model->RawPose());
   EXPECT_EQ("", model->PoseRelativeTo());
 
   EXPECT_TRUE(model->LinkNameExists("L"));
@@ -436,34 +444,28 @@ TEST(DOMFrame, LoadModelFramesAttachedTo)
   EXPECT_EQ(0u, model->JointCount());
   EXPECT_EQ(nullptr, model->JointByIndex(0));
 
-  EXPECT_EQ(5u, model->FrameCount());
+  EXPECT_EQ(4u, model->FrameCount());
   EXPECT_NE(nullptr, model->FrameByIndex(0));
   EXPECT_NE(nullptr, model->FrameByIndex(1));
   EXPECT_NE(nullptr, model->FrameByIndex(2));
   EXPECT_NE(nullptr, model->FrameByIndex(3));
-  EXPECT_NE(nullptr, model->FrameByIndex(4));
-  EXPECT_EQ(nullptr, model->FrameByIndex(5));
-  ASSERT_TRUE(model->FrameNameExists("F000"));
+  EXPECT_EQ(nullptr, model->FrameByIndex(4));
   ASSERT_TRUE(model->FrameNameExists("F00"));
   ASSERT_TRUE(model->FrameNameExists("F0"));
   ASSERT_TRUE(model->FrameNameExists("F1"));
   ASSERT_TRUE(model->FrameNameExists("F2"));
 
-  EXPECT_EQ("__model__", model->FrameByName("F000")->AttachedTo());
   EXPECT_TRUE(model->FrameByName("F00")->AttachedTo().empty());
   EXPECT_TRUE(model->FrameByName("F0")->AttachedTo().empty());
   EXPECT_EQ("L", model->FrameByName("F1")->AttachedTo());
   EXPECT_EQ("F1", model->FrameByName("F2")->AttachedTo());
 
-  EXPECT_TRUE(model->FrameByName("F000")->PoseRelativeTo().empty());
   EXPECT_TRUE(model->FrameByName("F00")->PoseRelativeTo().empty());
   EXPECT_TRUE(model->FrameByName("F0")->PoseRelativeTo().empty());
   EXPECT_TRUE(model->FrameByName("F1")->PoseRelativeTo().empty());
   EXPECT_TRUE(model->FrameByName("F2")->PoseRelativeTo().empty());
 
   std::string body;
-  EXPECT_TRUE(model->FrameByName("F000")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("L", body);
   EXPECT_TRUE(model->FrameByName("F00")->ResolveAttachedToBody(body).empty());
   EXPECT_EQ("L", body);
   EXPECT_TRUE(model->FrameByName("F0")->ResolveAttachedToBody(body).empty());
@@ -517,7 +519,7 @@ TEST(DOMFrame, LoadModelFramesInvalidAttachedTo)
   EXPECT_NE(std::string::npos,
     errors[9].Message().find(
       "PoseRelativeToGraph cycle detected, "
-      "already visited vertex [F4]"));
+      "already visited vertex [model_frame_invalid_attached_to::F4]"));
 
   // Try loading sdf::Model object
   sdf::SDFPtr sdfParsed(new sdf::SDF());
@@ -530,50 +532,22 @@ TEST(DOMFrame, LoadModelFramesInvalidAttachedTo)
   auto modelElem = rootElem->GetElement("model");
   ASSERT_NE(nullptr, modelElem);
 
-  // the first 10 errors from loading an sdf::Model match the first 10 errors
-  // from loading an sdf::Root
+  // there are no errors when loading an sdf::Model directly since it doesn't
+  // build graphs
   sdf::Model model;
   auto modelLoadErrors = model.Load(modelElem);
-  ASSERT_EQ(10u, modelLoadErrors.size());
-  for (int i = 0; i < 10; ++i)
-  {
-    EXPECT_EQ(errors[i].Code(), modelLoadErrors[i].Code());
-    EXPECT_EQ(errors[i].Message(), modelLoadErrors[i].Message());
-  }
+  EXPECT_EQ(0u, modelLoadErrors.size()) << modelLoadErrors;
 
   errors = model.ValidateGraphs();
-  EXPECT_EQ(6u, errors.size());
+  ASSERT_EQ(2u, errors.size()) << errors;
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
-      "FrameAttachedToGraph error, Non-LINK vertex with name [F3] is "
-      "disconnected; it should have 1 outgoing edge in MODEL attached_to "
-      "graph.")) << errors[0];
-  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
+      "FrameAttachedToGraph error: scope does not point to a valid graph"));
+  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[1].Message().find(
-      "Graph has __model__ scope but sink vertex named [F3] does not have "
-      "FrameType LINK or MODEL when starting from vertex with name [F3]"))
-      << errors[1];
-  EXPECT_EQ(errors[2].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[2].Message().find(
-      "cycle detected, already visited vertex [F4]")) << errors[2];
-  EXPECT_EQ(errors[3].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[3].Message().find(
-      "Vertex with name [F3] is disconnected; it should have "
-      "1 incoming edge in MODEL relative_to graph")) << errors[3];
-  EXPECT_EQ(errors[4].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[4].Message().find(
-      "frame with name [F3] is disconnected; its source vertex has name [F3], "
-      "but its source name should be __model__")) << errors[4];
-  EXPECT_EQ(errors[5].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[5].Message().find(
-      "PoseRelativeToGraph cycle detected, "
-      "already visited vertex [F4]")) << errors[5];
+      "PoseRelativeToGraph error: scope does not point to a valid graph"));
 }
 
 /////////////////////////////////////////////////
@@ -588,14 +562,14 @@ TEST(DOMFrame, LoadModelFramesAttachedToJoint)
   EXPECT_TRUE(root.Load(testFile).empty());
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_attached_to_joint", model->Name());
   EXPECT_EQ(2u, model->LinkCount());
   EXPECT_NE(nullptr, model->LinkByIndex(0));
   EXPECT_NE(nullptr, model->LinkByIndex(1));
   EXPECT_EQ(nullptr, model->LinkByIndex(2));
-  EXPECT_EQ(gz::math::Pose3d(0, 0, 0, 0, 0, 0), model->RawPose());
+  EXPECT_EQ(ignition::math::Pose3d(0, 0, 0, 0, 0, 0), model->RawPose());
   EXPECT_EQ("", model->PoseRelativeTo());
 
   EXPECT_TRUE(model->LinkNameExists("P"));
@@ -650,10 +624,11 @@ TEST(DOMFrame, LoadModelFramesAttachedToNestedModel)
 
   // Load the SDF file
   sdf::Root root;
-  EXPECT_TRUE(root.Load(testFile).empty());
+  sdf::Errors errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty()) << errors;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_attached_to_nested_model", model->Name());
   EXPECT_EQ(1u, model->LinkCount());
@@ -685,9 +660,9 @@ TEST(DOMFrame, LoadModelFramesAttachedToNestedModel)
 
   std::string body;
   EXPECT_TRUE(model->FrameByName("F1")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("nested_model", body);
+  EXPECT_EQ("nested_model::nested_link", body);
   EXPECT_TRUE(model->FrameByName("F2")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("nested_model", body);
+  EXPECT_EQ("nested_model::nested_link", body);
 }
 
 /////////////////////////////////////////////////
@@ -724,27 +699,23 @@ TEST(DOMFrame, LoadWorldFramesAttachedTo)
   ASSERT_TRUE(model->FrameNameExists("F0"));
   EXPECT_EQ("L", model->FrameByName("F0")->AttachedTo());
 
-  EXPECT_EQ(5u, world->FrameCount());
+  EXPECT_EQ(4u, world->FrameCount());
   EXPECT_NE(nullptr, world->FrameByIndex(0));
   EXPECT_NE(nullptr, world->FrameByIndex(1));
   EXPECT_NE(nullptr, world->FrameByIndex(2));
   EXPECT_NE(nullptr, world->FrameByIndex(3));
-  EXPECT_NE(nullptr, world->FrameByIndex(4));
-  EXPECT_EQ(nullptr, world->FrameByIndex(5));
+  EXPECT_EQ(nullptr, world->FrameByIndex(4));
   ASSERT_TRUE(world->FrameNameExists("world_frame"));
-  ASSERT_TRUE(world->FrameNameExists("F00"));
   ASSERT_TRUE(world->FrameNameExists("F0"));
   ASSERT_TRUE(world->FrameNameExists("F1"));
   ASSERT_TRUE(world->FrameNameExists("F2"));
 
   EXPECT_TRUE(world->FrameByName("world_frame")->AttachedTo().empty());
   EXPECT_TRUE(world->FrameByName("F0")->AttachedTo().empty());
-  EXPECT_EQ("world", world->FrameByName("F00")->AttachedTo());
   EXPECT_EQ("F0", world->FrameByName("F1")->AttachedTo());
   EXPECT_EQ("M1", world->FrameByName("F2")->AttachedTo());
 
   EXPECT_TRUE(world->FrameByName("world_frame")->PoseRelativeTo().empty());
-  EXPECT_TRUE(world->FrameByName("F00")->PoseRelativeTo().empty());
   EXPECT_TRUE(world->FrameByName("F0")->PoseRelativeTo().empty());
   EXPECT_TRUE(world->FrameByName("F1")->PoseRelativeTo().empty());
   EXPECT_TRUE(world->FrameByName("F2")->PoseRelativeTo().empty());
@@ -753,14 +724,12 @@ TEST(DOMFrame, LoadWorldFramesAttachedTo)
   EXPECT_TRUE(
     world->FrameByName("world_frame")->ResolveAttachedToBody(body).empty());
   EXPECT_EQ("world", body);
-  EXPECT_TRUE(world->FrameByName("F00")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("world", body);
   EXPECT_TRUE(world->FrameByName("F0")->ResolveAttachedToBody(body).empty());
   EXPECT_EQ("world", body);
   EXPECT_TRUE(world->FrameByName("F1")->ResolveAttachedToBody(body).empty());
   EXPECT_EQ("world", body);
   EXPECT_TRUE(world->FrameByName("F2")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("M1", body);
+  EXPECT_EQ("M1::L", body);
 }
 
 /////////////////////////////////////////////////
@@ -823,48 +792,22 @@ TEST(DOMFrame, LoadWorldFramesInvalidAttachedTo)
   auto worldElem = rootElem->GetElement("world");
   ASSERT_NE(nullptr, worldElem);
 
-  // the first 10 errors from loading an sdf::World match the first 10 errors
-  // from loading an sdf::Root
+  // there are no errors when loading an sdf::World directly since it doesn't
+  // build graphs
   sdf::World world;
   auto worldLoadErrors = world.Load(worldElem);
-  ASSERT_EQ(10u, worldLoadErrors.size());
-  for (int i = 0; i < 10; ++i)
-  {
-    EXPECT_EQ(errors[i].Code(), worldLoadErrors[i].Code());
-    EXPECT_EQ(errors[i].Message(), worldLoadErrors[i].Message());
-  }
+  EXPECT_EQ(0u, worldLoadErrors.size()) << worldLoadErrors;
 
   errors = world.ValidateGraphs();
-  EXPECT_EQ(6u, errors.size());
+  ASSERT_EQ(2u, errors.size()) << errors;
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
-      "FrameAttachedToGraph error, FRAME vertices in WORLD attached_to graph "
-      "should have 1 outgoing edge"));
-  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
+      "FrameAttachedToGraph error: scope does not point to a valid graph"));
+  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[1].Message().find(
-      "Graph has world scope but sink vertex named [F] does not have "
-      "FrameType WORLD or MODEL when starting from vertex with name [F]"));
-  EXPECT_EQ(errors[2].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[2].Message().find(
-      "cycle detected, already visited vertex [self_cycle]"));
-  EXPECT_EQ(errors[3].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[3].Message().find(
-      "MODEL / FRAME vertex with name [F] is disconnected; it should have "
-      "1 incoming edge in WORLD relative_to graph"));
-  EXPECT_EQ(errors[4].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[4].Message().find(
-      "frame with name [F] is disconnected; its source vertex has name [F], "
-      "but its source name should be world"));
-  EXPECT_EQ(errors[5].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[5].Message().find(
-      "PoseRelativeToGraph cycle detected, "
-      "already visited vertex [self_cycle]"));
+      "PoseRelativeToGraph error: scope does not point to a valid graph"));
 }
 
 /////////////////////////////////////////////////
@@ -878,10 +821,10 @@ TEST(DOMFrame, LoadModelFramesRelativeTo)
   sdf::Root root;
   EXPECT_TRUE(root.Load(testFile).empty());
 
-  using Pose = gz::math::Pose3d;
+  using Pose = ignition::math::Pose3d;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_relative_to", model->Name());
   EXPECT_EQ(1u, model->LinkCount());
@@ -1039,10 +982,10 @@ TEST(DOMFrame, LoadModelFramesRelativeToJoint)
   sdf::Root root;
   EXPECT_TRUE(root.Load(testFile).empty());
 
-  using Pose = gz::math::Pose3d;
+  using Pose = ignition::math::Pose3d;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_relative_to_joint", model->Name());
   EXPECT_EQ(2u, model->LinkCount());
@@ -1206,53 +1149,26 @@ TEST(DOMFrame, LoadWorldFramesRelativeTo)
   EXPECT_EQ("M2", world->ModelByName("M3")->PoseRelativeTo());
   EXPECT_EQ("F1", world->ModelByName("M4")->PoseRelativeTo());
 
-  EXPECT_EQ(5u, world->FrameCount());
+  EXPECT_EQ(4u, world->FrameCount());
   EXPECT_NE(nullptr, world->FrameByIndex(0));
   EXPECT_NE(nullptr, world->FrameByIndex(1));
   EXPECT_NE(nullptr, world->FrameByIndex(2));
   EXPECT_NE(nullptr, world->FrameByIndex(3));
-  EXPECT_NE(nullptr, world->FrameByIndex(4));
-  EXPECT_EQ(nullptr, world->FrameByIndex(5));
+  EXPECT_EQ(nullptr, world->FrameByIndex(4));
   ASSERT_TRUE(world->FrameNameExists("world_frame"));
   ASSERT_TRUE(world->FrameNameExists("F0"));
   ASSERT_TRUE(world->FrameNameExists("F1"));
-  ASSERT_TRUE(world->FrameNameExists("F1a"));
   ASSERT_TRUE(world->FrameNameExists("F2"));
 
   EXPECT_TRUE(world->FrameByName("world_frame")->PoseRelativeTo().empty());
   EXPECT_TRUE(world->FrameByName("F0")->PoseRelativeTo().empty());
   EXPECT_EQ("F0", world->FrameByName("F1")->PoseRelativeTo());
-  EXPECT_EQ("world", world->FrameByName("F1a")->PoseRelativeTo());
   EXPECT_EQ("M1", world->FrameByName("F2")->PoseRelativeTo());
 
   EXPECT_TRUE(world->FrameByName("world_frame")->AttachedTo().empty());
   EXPECT_TRUE(world->FrameByName("F0")->AttachedTo().empty());
   EXPECT_TRUE(world->FrameByName("F1")->AttachedTo().empty());
-  EXPECT_EQ("F1", world->FrameByName("F1a")->AttachedTo());
   EXPECT_TRUE(world->FrameByName("F2")->AttachedTo().empty());
-
-  using Pose = gz::math::Pose3d;
-  Pose pose;
-  EXPECT_TRUE(
-    world->FrameByName("world_frame")->
-      SemanticPose().Resolve(pose, "world").empty());
-  EXPECT_EQ(Pose(0, 0, 0, 0, 0, 0), pose);
-  EXPECT_TRUE(
-    world->FrameByName("F0")->
-      SemanticPose().Resolve(pose, "world").empty());
-  EXPECT_EQ(Pose(1, 0, 0, 0, 0, 0), pose);
-  EXPECT_TRUE(
-    world->FrameByName("F1")->
-      SemanticPose().Resolve(pose, "world").empty());
-  EXPECT_EQ(Pose(3, 0, 0, 0, 0, 0), pose);
-  EXPECT_TRUE(
-    world->FrameByName("F1a")->
-      SemanticPose().Resolve(pose, "world").empty());
-  EXPECT_EQ(Pose(4, 0, 0, 0, 0, 0), pose);
-  EXPECT_TRUE(
-    world->FrameByName("F2")->
-      SemanticPose().Resolve(pose, "world").empty());
-  EXPECT_EQ(Pose(3, 0, 0, 0, 0, 0), pose);
 }
 
 /////////////////////////////////////////////////
@@ -1268,7 +1184,7 @@ TEST(DOMFrame, LoadWorldFramesInvalidRelativeTo)
   for (auto e : errors)
     std::cout << e << std::endl;
   EXPECT_FALSE(errors.empty());
-  EXPECT_EQ(11u, errors.size());
+  EXPECT_EQ(15u, errors.size());
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_INVALID);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
@@ -1317,7 +1233,7 @@ TEST(DOMFrame, WorldIncludeModel)
   sdf::Errors errors = root.LoadSdfString(stream.str());
   EXPECT_TRUE(errors.empty()) << errors[0].Message();
 
-  gz::math::Pose3d expectedPoses[] = {
+  ignition::math::Pose3d expectedPoses[] = {
     {0, 0, 0, 0, 0, 0},
     {1, 0, 0, 0, 0, 0},
     {1, 1, 0, 0, 0, 0},
@@ -1331,85 +1247,9 @@ TEST(DOMFrame, WorldIncludeModel)
   {
     const sdf::Model *model = world->ModelByIndex(i);
     ASSERT_NE(nullptr, model);
-    gz::math::Pose3d modelPose;
+    ignition::math::Pose3d modelPose;
     sdf::Errors resolveErrors = model->SemanticPose().Resolve(modelPose);
-    EXPECT_TRUE(resolveErrors.empty());
+    EXPECT_TRUE(resolveErrors.empty()) << resolveErrors;
     EXPECT_EQ(expectedPoses[i], modelPose);
   }
 }
-
-////////////////////////////////////////
-// Test parsing a include element that has a pose element and includes a
-// submodel
-TEST(Frame, IncludeFrameWithSubmodel)
-{
-  const std::string MODEL_PATH =
-    sdf::filesystem::append(PROJECT_SOURCE_PATH, "test", "integration",
-                            "model", "box_with_submodel");
-
-  std::ostringstream stream;
-  std::string version = SDF_VERSION;
-  stream
-    << "<sdf version='" << version << "'>"
-    << "<world name='default'>"
-    << "<model name='top_level_model'>"
-    << "  <include>"
-    << "    <static>false</static>"
-    << "    <pose>5 5 0 0 0 0</pose>"
-    << "    <uri>" + MODEL_PATH +"</uri>"
-    << "  </include>"
-    << "</model>"
-    << "</world>"
-    << "</sdf>";
-
-  sdf::Root root;
-  sdf::Errors errors = root.LoadSdfString(stream.str());
-  EXPECT_TRUE(errors.empty()) << errors[0].Message();
-
-  const sdf::World *world = root.WorldByIndex(0);
-  ASSERT_NE(nullptr, world);
-  ASSERT_EQ(1u, world->ModelCount());
-
-  /* top level model: using include will merge top_level_model and box_with_submodel into:
-   *  <model name='top_level_model'>
-   *   <link name='box_with_submodel::link'>
-   *     <pose frame=''>5 5 0 0 -0 0</pose>  <<-- pose has been translated
-   *   </link>
-   *   <model name='box_with_submodel::submodel_box_with_submodel'>
-   *   ...
-   *   </model>
-   * </model>
-  */
-  const sdf::Model *model = world->ModelByIndex(0);
-  ASSERT_NE(nullptr, model);
-  EXPECT_EQ(model->Name(), "top_level_model");
-
-  const sdf::Link *link0 = model->LinkByIndex(0);
-  ASSERT_NE(nullptr, link0);
-  EXPECT_EQ(link0->Name(), "box_with_submodel::link");
-
-  gz::math::Pose3d linkPose;
-  sdf::Errors resolveErrors = link0->SemanticPose().Resolve(linkPose);
-  EXPECT_TRUE(resolveErrors.empty()) << resolveErrors[0].Message();
-  EXPECT_EQ(linkPose,
-            gz::math::Pose3d(5, 5, 0, 0, 0, 0));
-  /* submodel: pose from parent is translated to model. links are the same
-   * ...
-   *   <model name='box_with_submodel::submodel_box_with_submodel'>
-   *     <link name='submodel_link'>
-   *       <pose frame=''>0 0 0 0 -0 0</pose>
-   *     </link>
-   *     <pose frame=''>5 5 0 0 -0 0</pose>
-   *   </model>
-   */
-  const sdf::Model *submodel = model->ModelByIndex(0);
-  ASSERT_NE(nullptr, submodel);
-  EXPECT_EQ(submodel->Name(),
-            "box_with_submodel::submodel_of_box_with_submodel");
-
-  gz::math::Pose3d submodelPose;
-  resolveErrors = submodel->SemanticPose().Resolve(submodelPose);
-  EXPECT_TRUE(resolveErrors.empty()) << resolveErrors[0].Message();
-  EXPECT_EQ(submodelPose,
-            gz::math::Pose3d(5, 5, 0, 0, 0, 0));
-}
